/* eslint-disable no-param-reassign */
import { DEFAULT_SETTINGS } from './config.js';
import { getLocalStorage, setLocalStorage } from './utils.js';

const setFormItemValue = (id, value) => {
  document.querySelector(`#${id}`).value = value;
};

const setLoadingState = (btn) => {
  const btnText = btn.innerText;
  btn.disabled = true;
  btn.innerText = chrome.i18n.getMessage('settingsLoading');
  return btnText;
};

const resetButton = (btn, text) => {
  btn.disabled = false;
  btn.innerText = text;
};

async function saveSettings(form, settings) {
  await setLocalStorage({ settings });
  const elements = [...form.querySelectorAll('.form-control')];
  elements.forEach((el) => el.classList.add('is-valid'));

  setTimeout(() => {
    elements.forEach((el) => el.classList.remove('is-valid'));
  }, 2000);
}

async function loadSettings() {
  const result = await getLocalStorage('settings');
  const settings = { ...DEFAULT_SETTINGS, ...result };

  setFormItemValue('prompt', settings.prompt);
  setFormItemValue('token', settings.token);
  setFormItemValue('host', settings.host);
  setFormItemValue('model', settings.model);
}

function changeButton(form) {
  const btn = form.querySelector('button[type="submit"]');
  const btnText = setLoadingState(btn);
  setTimeout(() => resetButton(btn, btnText), 300);
}

async function validateToken({ token, host, model }) {
  try {
    const response = await fetch(`${host}/v1/chat/completions`, {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json',
        Authorization: `Bearer ${token}`,
      },
      body: JSON.stringify({
        model,
        stream: true,
      }),
    });
    const data = await response.json();
    if ([200, 400].includes(response.status)) return { verified: true };
    return { verified: false, message: data.error.message };
  } catch (error) {
    return { verified: false, message: error.message };
  }
}

document.addEventListener('DOMContentLoaded', loadSettings);

document
  .querySelector('#form-prompt')
  .addEventListener('submit', async (event) => {
    event.preventDefault();

    const result = await getLocalStorage('settings');
    const settings = {
      ...DEFAULT_SETTINGS,
      ...result,
      prompt: document.querySelector('#prompt').value,
    };

    changeButton(event.target);
    saveSettings(event.target, settings);
  });

document
  .querySelector('#form-api')
  .addEventListener('submit', async (event) => {
    event.preventDefault();

    const form = event.target;
    const token = form.querySelector('#token').value;
    let host = form.querySelector('#host').value.trim().replace(/\/+$/, '');
    const model = form.querySelector('#model').value;

    if (!host) {
      host = DEFAULT_SETTINGS.host;
      form.querySelector('#host').value = DEFAULT_SETTINGS.host;
    }

    form.querySelector('.invalid-feedback')?.remove();
    [...form.querySelectorAll('.form-control')].forEach((el) => {
      el.classList.remove('is-invalid');
    });

    const btn = form.querySelector('button[type="submit"]');
    const btnText = setLoadingState(btn);

    const { verified, message } = await validateToken({ token, host, model });
    const result = await getLocalStorage('settings');

    if (verified) {
      const settings = {
        ...DEFAULT_SETTINGS,
        ...result,
        token,
        host,
        model,
      };

      saveSettings(form, settings);
    } else {
      [...form.querySelectorAll('.form-control')].forEach((el) => {
        el.classList.add('is-invalid');
      });
      document
        .querySelector('#model')
        .insertAdjacentHTML(
          'afterend',
          `<div class="invalid-feedback">${message}</div>`,
        );
    }

    resetButton(btn, btnText);
  });

document
  .querySelector('#toggle-password')
  .addEventListener('click', ({ currentTarget }) => {
    const icon = currentTarget.querySelector('.bi');
    icon.classList.toggle('bi-eye');
    icon.classList.toggle('bi-eye-slash');

    const input = document.querySelector('#token');
    input.type = input.type === 'password' ? 'text' : 'password';
  });

document.querySelector('#button-reset-prompt').addEventListener('click', () => {
  setFormItemValue('prompt', DEFAULT_SETTINGS.prompt);
  document.querySelector('#form-prompt').requestSubmit();
});

document.title = chrome.i18n.getMessage('appName');
try {
  [...document.querySelectorAll('[data-locale]')].forEach((element) => {
    const text = chrome.i18n.getMessage(element.dataset.locale);
    if (text) element.innerText = text;
  });

  [...document.querySelectorAll('[data-locale-html]')].forEach((element) => {
    const text = chrome.i18n.getMessage(element.dataset.localeHtml);
    if (text) element.innerHTML = text;
  });
} catch (error) {
  console.error('[GPT_Summarizer]:', error);
}

document.body.style.opacity = 1;
